﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace database1.pages {
	public partial class ActionQueries : System.Web.UI.Page {
		protected void Page_Load(object sender, EventArgs e) {
			if (!Page.IsPostBack) {
				// Display Players and Teams by calling db selection
				// radio button changed event.
				rblDB.Items[0].Selected = true;
				rblDB_SelectedIndexChanged(sender, e);
			}
			lblPlayerInsert.Visible = false;
			lblTeamInsert.Visible = false;
			lblPlayerUpdate.Visible = false;
			lblPlayerDelete.Visible = false;
			lblTeamDelete.Visible = false;
		}

		#region Event Handlers
		protected void rblDB_SelectedIndexChanged(object sender, EventArgs e) {
			DisplayGridView(gvPlayers, "Players", rblDB.SelectedValue);
			DisplayGridView(gvTeams, "Teams", rblDB.SelectedValue);
		}

		protected void btnInsertTeam_Click(object sender, EventArgs e) {
			IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
			GenerateInsertTeamSQL(cmd);

			try {
				cmd.Connection.Open();

				txtMsg.Text += "\r\n***Attempting to insert Team into: " + rblDB.SelectedValue + " database\r\n";
				txtMsg.Text += "IDbConnection.State: " + cmd.Connection.State.ToString() + "\r\n";

				int num = cmd.ExecuteNonQuery();
				cmd.Connection.Close();

				DisplayGridView(gvTeams, "Teams", rblDB.SelectedValue);
				lblTeamInsert.Text = "Insert Successful";
				lblTeamInsert.Visible = true;
				txtMsg.Text = "Teams added: " + num + "\r\n";
			}
			catch (Exception ex) {
				lblTeamInsert.Text = "Insert Failed";
				lblTeamInsert.Visible = true;
				txtMsg.Text += "\r\nTeam insertion error\r\n";
				txtMsg.Text += ex.ToString();
			}
		}

		protected void btnInsertPlayer_Click(object sender, System.EventArgs e) {
			IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
			GenerateInsertPlayerSQL(cmd);

			try {
				cmd.Connection.Open();

				txtMsg.Text = "Attempting to insert into: " + rblDB.SelectedValue + " database\r\n";
				txtMsg.Text += "IDbConnection.State: " + cmd.Connection.State.ToString() + "\r\n";

				int num = cmd.ExecuteNonQuery();
				cmd.Connection.Close();

				DisplayGridView(gvPlayers, "Players", rblDB.SelectedValue);
				lblPlayerInsert.Text = "Insert Successful";
				lblPlayerInsert.Visible = true;
				txtMsg.Text = "Players added: " + num + "\r\n";
			}
			catch (Exception ex) {
				lblPlayerInsert.Text = "Insert Failed";
				lblPlayerInsert.Visible = true;
				txtMsg.Text = "Error adding player, TeamID probably incorrect\r\n";
				txtMsg.Text += ex.ToString();
			}
		}

		protected void btnUpdatePlayer_Click(object sender, System.EventArgs e) {
			IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
			GenerateUpdatePlayerSQL(cmd);

			try {
				cmd.Connection.Open();

				txtMsg.Text = "Attempting to update in: " + rblDB.SelectedValue + " database\r\n";
				txtMsg.Text += "IDbConnection.State: " + cmd.Connection.State.ToString() + "\r\n";

				int num = cmd.ExecuteNonQuery();
				cmd.Connection.Close();

				DisplayGridView(gvPlayers, "Players", rblDB.SelectedValue);
				lblPlayerUpdate.Text = "Update Successful";
				lblPlayerUpdate.Visible = true;
				txtMsg.Text += "Players updated: " + num + "\r\n";
			}
			catch (Exception ex) {
				lblPlayerUpdate.Text = "Update Failed";
				lblPlayerUpdate.Visible = true;
				txtMsg.Text += "\r\nPlayer update error\r\n";
				txtMsg.Text += ex.ToString();
			}
		}

		protected void btnDeletePlayer_Click(object sender, System.EventArgs e) {
			IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
			GenerateDeletePlayerSQL(cmd);

			try {
				cmd.Connection.Open();

				txtMsg.Text = "Attempting to delete player from: " + rblDB.SelectedValue + " database\r\n";
				txtMsg.Text += "IDbConnection.State: " + cmd.Connection.State.ToString() + "\r\n";

				int num = cmd.ExecuteNonQuery();
				cmd.Connection.Close();

				txtMsg.Text += "Players deleted: " + num + "\r\n";

				if (num == 0) {
					txtMsg.Text += "***Delete Failed. Invalid PlayerID\r\n";
					lblPlayerDelete.Text = "Delete Failed. Invalid PlayerID";
					lblPlayerDelete.Visible = true;
				}
				else {
					DisplayGridView(gvPlayers, "Players", rblDB.SelectedValue);
					lblPlayerDelete.Text = "Delete Successful";
					lblPlayerDelete.Visible = true;
				}
			}
			catch (Exception ex) {
				lblPlayerDelete.Text = "Delete Failed. Unknown Error";
				lblPlayerDelete.Visible = true;
				txtMsg.Text = "Delete Failed. Unknown Error\r\n";
				txtMsg.Text += ex.ToString();
			}
		}

		protected void btnDeleteTeam_Click(object sender, EventArgs e) {
			IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
			GenerateDeleteTeamSQL(cmd);

			try {
				cmd.Connection.Open();

				txtMsg.Text = "Attempting to delete team from: " + rblDB.SelectedValue + " database\r\n";
				txtMsg.Text += "IDbConnection.State: " + cmd.Connection.State.ToString() + "\r\n";

				int num = cmd.ExecuteNonQuery();
				cmd.Connection.Close();

				txtMsg.Text += "Teams deleted: " + num + "\r\n";

				if (num == 0) {
					lblTeamDelete.Text = "Delete Failed. Invalid TeamID";
					lblTeamDelete.Visible = true;
				}
				else {
					lblTeamDelete.Text = "Delete Successful";
					lblTeamDelete.Visible = true;
					DisplayGridView(gvTeams, "Teams", rblDB.SelectedValue);
				}
			}
			catch (Exception ex) {
				lblTeamDelete.Text = "Delete Failed. Team has Players";
				lblTeamDelete.Visible = true;
				txtMsg.Text = "Delete Failed. Players exist on Team\r\n";
				txtMsg.Text += ex.ToString();
			}

		}

		protected void btnShowHide_Click(object sender, EventArgs e) {
			if (btnShowHide.Text.Equals("Show Data")) {
				btnShowHide.Text = "Hide Data";
				Panel4.Visible = true;
			}
			else {
				btnShowHide.Text = "Show Data";
				Panel4.Visible = false;
			}
		}

		protected void btnClearFields_Click(object sender, EventArgs e) {
			clearTextBoxes();
		}
		#endregion

		#region Generate SQL methods

		private string GetSelectPlayersSQL() {
			String sql =
				"SELECT " +
					"Players.PlayerID, " +
					"Players.TeamID, " +
					"Players.LName, " +
					"Players.FName, " +
					"Players.PNumber, " +
					"Players.BDate " +
				"FROM " +
					"Players " +
				"ORDER BY " +
					"Players.LName Asc, " +
					"Players.FName Asc";

			return sql;
		}

		private string GetSelectTeamsSQL() {
			String sql =
				"SELECT " +
					"Teams.TeamID, " +
					"Teams.Name, " +
					"Teams.CoachLName, " +
					"Teams.CoachFName " +
				"FROM " +
					"Teams " +
				"ORDER BY " +
					"Teams.Name ASC";
			return sql;
		}

		private void GenerateInsertTeamSQL(IDbCommand cmd) {
			cmd.CommandText = "Insert Into " +
				"Teams " +
				"( Name, CoachFName, CoachLName ) " +
				"Values " +
				"( @Name, @CoachFName, @CoachLName )";

			// Clear parameters and add new ones.
			cmd.Parameters.Clear();
			addParameter("@Name", txtTeamName.Text, cmd);
			addParameter("@CoachFName", txtCoachLName.Text, cmd);
			addParameter("@CoachLName", txtCoachFName.Text, cmd);
		}

		private void GenerateInsertPlayerSQL(IDbCommand cmd) {
			cmd.CommandText = "Insert Into " +
				"Players " +
				"( TeamID, LName, FName, PNumber, BDate ) " +
				"Values " +
				"( @TeamID, @LName, @FName, @PNumber, @BDate )";

			// Clear any parameters from the command.
			cmd.Parameters.Clear();
			addParameter("@TeamID", txtTeamID_insert.Text, cmd);
			addParameter("@LName", txtLName_insert.Text, cmd);
			addParameter("@FName", txtFName_insert.Text, cmd);
			addParameter("@PNumber", txtJerNum_insert.Text, cmd);
			addParameter("@BDate", txtBDate_insert.Text, cmd);
		}

		private void GenerateUpdatePlayerSQL(IDbCommand cmd) {
			cmd.CommandText = "UPDATE " +
				"Players " +
				"Set " +
				"TeamID=@TeamID, " +
				"LName=@LName, " +
				"FName=@FName, " +
				"PNumber=@PNumber, " +
				"BDate=@BDate " +
				"Where " +
				"PlayerID=@PlayerID";

			cmd.Parameters.Clear();
			addParameter("@TeamID", txtTeamID_update.Text, cmd);
			addParameter("@LName", txtLName_update.Text, cmd);
			addParameter("@FName", txtFName_update.Text, cmd);
			addParameter("@PNumber", txtJerNum_update.Text, cmd);
			addParameter("@BDate", txtBDate_update.Text, cmd);
			addParameter("@PlayerID", txtPlayerID_update.Text, cmd);
		}

		private void GenerateDeletePlayerSQL(IDbCommand cmd) {
			// Set delete SQL statement.
			cmd.CommandText = "Delete From " +
				"Players " +
				"Where " +
				"PlayerID=@PlayerID";

			cmd.Parameters.Clear();
			addParameter("@PlayerID", txtPlayerIDDelete.Text, cmd);
		}

		private void GenerateDeleteTeamSQL(IDbCommand cmd) {
			// Set delete SQL statement.
			cmd.CommandText = "Delete From " +
				"Teams " +
				"Where " +
				"TeamID=@TeamID";

			cmd.Parameters.Clear();
			addParameter("@TeamID", txtTeamIDDelete.Text, cmd);
		}

		private void addParameter(string paramName, string paramValue, IDbCommand cmd) {
			// Create a parameter.
			IDbDataParameter param = cmd.CreateParameter();
			// Name the parameter.
			param.ParameterName = paramName;
			// Assign a value to the parameter.
			param.Value = paramValue;
			// Add the parameter to the command.
			cmd.Parameters.Add(param);
		}
		#endregion

		#region Helper methods
		private void DisplayGridView(GridView gv, string table, string dbType) {
			IDbCommand cmd = ConnectionFactory.GetCommand(dbType);
			if (table.Equals("Players"))
				cmd.CommandText = GetSelectPlayersSQL();
			else if (table.Equals("Teams"))
				cmd.CommandText = GetSelectTeamsSQL();

			try {
				cmd.Connection.Open();
				IDataReader dr = cmd.ExecuteReader();

				gv.DataSource = dr;
				gv.DataBind();

				dr.Close();
				cmd.Connection.Close();
			}
			catch (Exception ex) {
				txtMsg.Text += "\r\n" + table + " GridView Display error\r\n";
				txtMsg.Text += ex.ToString();
			}
		}

		private void clearTextBoxes() {
			clearTextBoxes(Page.Controls);
		}

		// Clears all text boxes on form. Don't understand code.
		private void clearTextBoxes(ControlCollection controlCollection) {
			foreach (Control control in controlCollection) {
				//txtMsg.Text += control.GetType().ToString() + "\n";
				if (control is TextBox) {
					((TextBox)control).Text = "";
					//txtMsg.Text += "-->Found TextBox" + "\n";
				}
				if (control.Controls != null) {
					clearTextBoxes(control.Controls);
					//txtMsg.Text += "-->Found Something non-null" + "\n";
				}
			}
		}
		#endregion
	}
}