﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace database1.pages {
	public partial class Read : System.Web.UI.Page {
		protected void Page_Load(object sender, EventArgs e) {
			try {
				IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
				cmd.CommandText = getSQL();
				cmd.Connection.Open();
				IDataReader dr = cmd.ExecuteReader();

				txtMsg.Text = "Attempting to read from: " + rblDB.SelectedValue + " database" + Environment.NewLine;
				txtMsg.Text += "IDbConnection.State: " + cmd.Connection.State.ToString() + Environment.NewLine; ;
				txtMsg.Text += "IDataReader.IsClosed: " + dr.IsClosed + Environment.NewLine;
				txtMsg.Text += "cmd.CommandText: " + cmd.CommandText + Environment.NewLine + Environment.NewLine;

				/*
				 * Uses the BuildPlayerList method to read from the database and return a List of Player objects.
				 */
				List<Player> players = BuildPlayersList(dr);

				dr.Close();
				cmd.Connection.Close();

				/*
				 * Uses the player's list to calculate the average age via a helper method, GetAverageAge.
				 */
				double avgAge = GetAverageAge(players);
				txtMsg.Text += String.Format("Average age of players: {0:0.0}", avgAge) + Environment.NewLine;

				/* 
				 * Uses a helper method, DisplayPlayersList to display the list of players 
				 */
				DisplayPlayersList(players);
			}
			catch (Exception ex) {
				txtMsg.Text += "\r\nError in SelectedIndexChanged\r\n";
				txtMsg.Text += ex.ToString();
			}
		}

		private String getSQL() {
			String sql =
				"SELECT " +
					"Players.PlayerID, " +
					"Players.TeamID, " +
					"Players.LName, " +
					"Players.FName, " +
					"Players.PNumber, " +
					"Players.BDate " +
				"FROM " +
					"Players " +
				"ORDER BY " +
					"Players.LName Asc, " +
					"Players.FName Asc";

			return sql;

		}

		private List<Player> BuildPlayersList(IDataReader dr) {
			List<Player> players = new List<Player>();

			// Read the data from the data reader. Note that this is one-pass, forward only.
			while (dr.Read()) {
				int pID = dr.GetInt32(0);
				int tID = dr.GetInt32(1);
				string lName = dr.GetString(2);
				string fName = dr.GetString(3);
				int jerNum = dr.GetInt32(4);
				DateTime bDate = dr.GetDateTime(5);

				Player p = new Player(pID, tID, lName, fName, jerNum, bDate);
				players.Add(p);
			}
			return players;
		}

		private void DisplayPlayersList(List<Player> players) {
			foreach (Player p in players) {
				txtMsg.Text += p.PlayerID + " " +
							   p.TeamID + " " +
							   p.LastName + " " +
							   p.FirstName + " " +
							   p.JerseyNum + " " +
							   p.BirthDate.ToShortDateString() +
							   Environment.NewLine;
			}
		}

		private double GetAverageAge(List<Player> players) {
			double sum = 0.0;

			foreach (Player p in players) {
				DateTime bDay = p.BirthDate;

				TimeSpan span = DateTime.Today.Subtract(bDay);
				double years = span.TotalDays / 365.25;
				sum += years;
			}
			return sum / players.Count;
		}

		protected void btnDisplay_Click(object sender, EventArgs e) {
			try {
				IDbCommand cmd = ConnectionFactory.GetCommand(rblDB.SelectedValue);
				cmd.CommandText = getSQL();
				cmd.Connection.Open();
				IDataReader dr = cmd.ExecuteReader();

				gvPlayers.DataSource = dr;
				gvPlayers.DataBind();

				dr.Close();
				cmd.Connection.Close();
			}
			catch (Exception ex) {
				txtMsg.Text += "\r\nGridView Display error\r\n";
				txtMsg.Text += ex.ToString();
			}
		}
	}
}